@@ -7,9 +7,24 @@ _RECORDSIZE = 4
 
 
 class BinaryReader(abc.ABC):
+    """
+    Parent class for reading the binary output (.out)- and hotstart-files.
+
+    Attributes:
+        fp (file-like): Stream of the open file.
+        filename (str): Path to the .out-file.
+
+    Args:
+        filename (str): Path to the .out-file.
+    """
     def __init__(self, filename):
         self.fp = None
-        self.fp = open(filename, "rb")
+        if all([hasattr(filename, i) for i in ['tell', 'seek', 'read', 'close']]):
+            self.fp = filename
+            self.filename = '<stream>'
+        else:
+            self.filename = filename
+            self.fp = open(filename, "rb")
 
     @abc.abstractmethod
     def __repr__(self):
@@ -22,26 +37,24 @@ class BinaryReader(abc.ABC):
         self.close()
 
     def close(self):
+        """
+        Close the .out-file.
+
+        Close the binary file to prevent memory errors.
+        """
         if self.fp is not None:
             self.fp.close()
 
     def delete(self):
+        """
+        Close and delete the .out-file.
+        """
         self.close()
         remove(self.filename)
 
     def __del__(self):
         self.close()
 
-    @property
-    def filename(self):
-        """
-        path and filename of the .out-file
-
-        Returns:
-            str: path and filename of the .out-file
-        """
-        return self.fp.name
-
     def _set_position(self, offset, whence=SEEK_SET):
         if self.fp.tell() != offset:
             self.fp.seek(offset, whence)
