@@ -1,11 +1,14 @@
+import abc
+import datetime
 import re
 import types
 from abc import ABC
-from inspect import isfunction, isclass
-
+from inspect import isfunction, isclass, getdoc, signature
+import warnings
 from numpy import isnan
 from pandas import DataFrame, Series
 from tqdm.auto import tqdm
+from pprint import pformat
 
 from ._type_converter import type2str, is_equal, txt_to_lines
 from .section_labels import *
@@ -13,13 +16,27 @@ from .section_lists import LINK_SECTIONS, NODE_SECTIONS
 
 SWMM_VERSION = '5.1.015'
 
-SEP_INP = ';;' + "_" * 100
+COMMENT_STR = ';;'
+SEP_INP = COMMENT_STR + "_" * 100
+COMMENT_EMPTY_SECTION = COMMENT_STR + ' No Data'
+
+
+class SwmmInputWarning(UserWarning):
+    pass
+
+
+def head_to_str(head):
+    return f'\n\n{SEP_INP}\n[{head}]\n'
+
+
+_TYPES_NO_COPY = (int, float, str, datetime.date, datetime.time)
 
 
 ########################################################################################################################
 class CustomDict:
-    """imitate :class:`collections.UserDict` (:term:`dict-like <mapping>`) but operations only effect self._data"""
-
+    """
+    imitate :class:`collections.UserDict` (:term:`dict-like <mapping>`) but operations only effect self._data
+    """
     def __init__(self, d=None, **kwargs):
         if d is None:
             self._data = kwargs
@@ -48,18 +65,29 @@ class CustomDict:
         return self._data.__contains__(key)
 
     def __repr__(self):
-        return self._data.__repr__()
+        return pformat(self._data, indent=4, sort_dicts=False)
+        # return self._data.__repr__()
 
     def __str__(self):
         return self._data.__str__()
 
     def get(self, key, default=None):
+        """
+
+        Args:
+            key:
+            default:
+
+        Returns:
+
+        """
         if isinstance(key, list):
             return (self.get(k) for k in key)
         return self._data.get(key) if key in self else default
 
     def copy(self):
-        return type(self)(self._data.copy())
+        return type(self)({k: v if isinstance(v, _TYPES_NO_COPY) else v.copy() for k, v in self._data.items()})
+        # return type(self)(self._data.copy())
 
     def values(self):
         return self._data.values()
@@ -84,29 +112,22 @@ class CustomDict:
     #     return id(self)
 
 
-class CustomDictWithAttributes(CustomDict):
-    def copy(self):
-        new = type(self)()
-        for key in self:
-            if getattr(self[key], 'copy', False):
-                new[key] = self[key].copy()
-            else:
-                new[key] = self[key]
-        return new
-
-
 ########################################################################################################################
-class InpSectionGeneric(CustomDictWithAttributes):
+class InpSectionGeneric(CustomDict):
     """
     abstract class for ``.inp``-file sections without objects
 
-    :term:`dict-like <mapping>`"
+    Notes:
+        Acts :term:`dict-like <mapping>`
+
+    Attributes:
+        _label (str): label of the section
     """
     _label = ''
     """str: label of the section"""
 
     def __init__(self, *args, **kwargs):
-        CustomDictWithAttributes.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._inp = None
 
     def __setitem__(self, key, item):
@@ -120,51 +141,62 @@ class InpSectionGeneric(CustomDictWithAttributes):
         super().__delitem__(key)
 
     def set_parent_inp(self, inp):
+        """
+        Set inp-data object related to this section.
+
+        Args:
+            inp (SwmmInput): inp-data object related to this section.
+        """
         self._inp = inp
 
     def get_parent_inp(self):
+        """
+        Get inp-data object related to this section.
+
+        Returns:
+            SwmmInput: inp-data object related to this section.
+        """
         return self._inp
 
     @classmethod
     def from_inp_lines(cls, lines):
         """
-        read ``.inp``-file lines and create an section object
+        Read ``.inp``-file lines and create a new section.
 
         Args:
-            lines (str | list[list[str]]): lines in the section of the ``.inp``-file
+            lines (str | list[list[str]]): Lines in the section of the ``.inp``-file
 
         Returns:
-            InpSectionGeneric: object of the section
+            InpSectionGeneric: New section.
         """
         pass
 
-    def to_inp_lines(self, fast=False):
+    def to_inp_lines(self, fast=False, sort_objects_alphabetical=False):
         """
-        write ``.inp``-file lines of the section object
+        Convert the section to ``.inp``-file.lines.
 
         Args:
             fast (bool): speeding up conversion
 
-                - :obj:`True`: if no special formation of the input file is needed
-                - :obj:`False`: section is converted into a table to prettify string output (slower)
+                - :obj:`True`: If no special formation of the input file is needed.
+                - :obj:`False`: Section is converted into a table to prettify string output (slower).
 
         Returns:
-            str: ``.inp``-file lines of the section object
+            str: Lines of the ``.inp``-file section.
         """
-        f = ''
-        max_len = len(max(self.keys(), key=len)) + 2
-        for sub in self:
-            f += '{key}{value}'.format(key=sub.ljust(max_len),
-                                       value=type2str(self[sub]) + '\n')
-        return f
+        # size of the longest key (number of characters)
+        max_len = len(max(self.keys(), key=len))
+        return '\n'.join(f'{(key if isinstance(key, str) else " ".join(key)).ljust(max_len)}  {type2str(value)}' for key, value in self.items())
 
     @classmethod
     def create_section(cls):
-        return cls()
+        """
+        Create a new empty section.
 
-    # @property
-    # def id(self):
-    #     return id(self)
+        Returns:
+            cls(): Empty section.
+        """
+        return cls()
 
 
 ########################################################################################################################
@@ -182,14 +214,26 @@ class InpSection(CustomDict):
                 This information is used to set the index of the section and
                 to decide if the section can be exported (converted to a string) as a table.
         """
-        CustomDict.__init__(self)
+        super().__init__()
         self._section_object = section_object
         self._inp = None
 
     def set_parent_inp(self, inp):
+        """
+        Set inp-data object related to this section.
+
+        Args:
+            inp (SwmmInput): inp-data object related to this section.
+        """
         self._inp = inp
 
     def get_parent_inp(self):
+        """
+        Get inp-data object related to this section.
+
+        Returns:
+            SwmmInput: inp-data object related to this section.
+        """
         return self._inp
 
     # def __repr__(self):
@@ -202,13 +246,19 @@ class InpSection(CustomDict):
     def __setitem__(self, key, value):
         if isinstance(self._identifier, str):
             if not isinstance(key, str):
-                raise UserWarning('Wrong key type')
+                warnings.warn(f'Wrong key type (Unknown Behaviour)\n'
+                              f'Needed: "{self._identifier}" as string\n'
+                              f'Given: {key} of type {type(key)}', SwmmInputWarning)
         else:
             if len(self._identifier) != len(key):
-                raise UserWarning('Wrong number of keys')
+                warnings.warn(f'Wrong number of keys (Unknown Behaviour)\n'
+                              f'Needed keys: "{self._identifier}"\n'
+                              f'Given keys: "{key}"', SwmmInputWarning)
 
         if not isinstance(value, self._section_object):
-            raise UserWarning('Wrong section-object type')
+            warnings.warn(f'Wrong section-object type (Unknown Behaviour)\n'
+                          f'Needed type: "{self._section_object}"\n'
+                          f'Given type: "{type(value)}"', SwmmInputWarning)
 
         super().__setitem__(key, value)
 
@@ -226,11 +276,19 @@ class InpSection(CustomDict):
     def _identifier(self):
         """
         to set the index of the section (key to select an object an index for the dataframe export)
+
         Returns:
             str | tuple: key of the objects label (can be a single or multiple keys)
         """
         return self._section_object._identifier
 
+    @property
+    def _index_labels(self):
+        if isinstance(self._identifier, tuple):
+            return list(self._identifier)
+        else:
+            return self._identifier
+
     @property
     def _label(self):
         """
@@ -319,10 +377,28 @@ class InpSection(CustomDict):
              str: lines of the ``.inp``-file section
         """
         if not self:  # if empty
-            return ';; No Data'
+            return COMMENT_EMPTY_SECTION
 
         if fast or not self._table_inp_export:
+            return '\n'.join(self.iter_inp_lines(sort_objects_alphabetical)) + '\n'
+        else:
+            return dataframe_to_inp_string(self.get_dataframe(set_index=True,
+                                                              sort_objects_alphabetical=sort_objects_alphabetical))
 
+    def iter_inp_lines(self, sort_objects_alphabetical=False):
+        """
+        convert the section to a multi-line ``.inp``-file conform string
+
+        This function is used for the ``.inp``-file writing
+
+        Args:
+            sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
+                default: use order of the read inp-file and append new objects
+
+        Yields:
+             str: lines of the ``.inp``-file section
+        """
+        if self:
             # only show write progress for big files
             n_objects = len(self.keys())
             values = self.get_objects(sort_objects_alphabetical)
@@ -334,10 +410,10 @@ class InpSection(CustomDict):
             else:
                 _iterable = values
 
-            return '\n'.join(o.to_inp_line() for o in _iterable)
-        else:
-            return dataframe_to_inp_string(self.get_dataframe(set_index=True,
-                                                              sort_objects_alphabetical=sort_objects_alphabetical))
+            for o in _iterable:
+                yield o.to_inp_line()
+        else:  # if empty
+            yield COMMENT_EMPTY_SECTION
 
     @property
     def frame(self):
@@ -365,7 +441,7 @@ class InpSection(CustomDict):
             return DataFrame()
         df = DataFrame([i.to_dict_() for i in self.get_objects(sort_objects_alphabetical)])
         if set_index:
-            df = df.set_index(self._identifier)
+            df = df.set_index(self._index_labels)
         return df
 
     def create_new_empty(self):
@@ -388,7 +464,8 @@ class InpSection(CustomDict):
         # ΔTime: 18.678 s
         # new._data = deepcopy(self._data)
         # ΔTime: 2.943 s
-        new._data = {k: self[k].copy() for k in self}
+        # new._data = {k: self[k].copy() for k in self}
+        new.add_multiple(v.copy() for v in self.values())
         return new
 
     def filter_keys(self, keys, by=None):
@@ -412,13 +489,18 @@ class InpSection(CustomDict):
                 return tuple()
 
             if isinstance(by, (list, set, tuple)):
-                filtered_keys = f[f[by].isin(keys).all(axis=1)].set_index(self._identifier).index
+                if (set(f.columns) & set(by)) != set(by):
+                    raise SwmmInputWarning(f'{set(by) - set(f.columns)} not available. possible={f.columns.tolist()}')
+
+                f_filtered = f[by].isin(keys).all(axis=1)
                 # filtered_keys = (k for k in self if any(map(lambda b: self[k][b] in keys, by)))
 
             else:
                 # filtered_keys = filter(lambda k: self[k][by] in keys, self)
                 # filtered_keys = (k for k in self if self[k][by] in keys)
-                filtered_keys = f[f[by].isin(keys)].set_index(self._identifier).index
+                f_filtered = f[by].isin(keys)
+
+            filtered_keys = f[f_filtered].set_index(self._index_labels).index
 
         return (self[k] for k in filtered_keys)
 
@@ -438,6 +520,64 @@ class InpSection(CustomDict):
         return new
 
 
+class InpSectionGeo(InpSection):
+    """child class of :class:`~swmm_api.input_file.helpers.InpSection`. See parent class for all functions."""
+    def __init__(self, section_object, crs="EPSG:32633"):
+        """
+        create a section for ``.inp``-file with geo-objects (i.e. nodes, links, subcatchments, raingages, ...)
+
+        Args:
+            section_object (BaseSectionObject-like): object class which is stored in this section.
+                This information is used to set the index of the section and
+                to decide if the section can be exported (converted to a string) as a table.
+            crs: Coordinate Reference System of the geometry objects.
+                Can be anything accepted by :meth:`pyproj.CRS.from_user_input() <pyproj.crs.CRS.from_user_input>`,
+                such as an authority string (eg “EPSG:32633”) or a WKT string.
+        """
+        InpSection.__init__(self, section_object)
+        self._crs = crs
+
+    def set_crs(self, crs):
+        """
+        Set the Coordinate Reference System (CRS) of a geo-section.
+
+        Notes:
+            The underlying geometries are not transformed to this CRS.
+
+        Args:
+            crs: Coordinate Reference System of the geometry objects.
+                Can be anything accepted by :meth:`pyproj.CRS.from_user_input() <pyproj.crs.CRS.from_user_input>`,
+                such as an authority string (eg “EPSG:32633”) or a WKT string.
+        """
+        self._crs = crs
+
+    @property
+    def geo_series(self):
+        """
+        Get a geopandas.GeoSeries representation for the geo-section.
+        This function sets the object default crs.
+
+        Returns:
+            geopandas.GeoSeries: geo-series of the section-data
+        """
+        return self.get_geo_series(self._crs)
+
+    def get_geo_series(self, crs):
+        """
+        get a geopandas.GeoSeries representation for the geo-section using a custom crs.
+
+        Args:
+            crs: Coordinate Reference System of the geometry objects.
+                Can be anything accepted by :meth:`pyproj.CRS.from_user_input() <pyproj.crs.CRS.from_user_input>`,
+                such as an authority string (eg “EPSG:32633”) or a WKT string.
+
+        Returns:
+            geopandas.GeoSeries: geo-series of the section-data
+        """
+        from geopandas import GeoSeries
+        return GeoSeries({label: item.geo for label, item in self.items()}, crs=crs, name='geometry')
+
+
 def split_line_with_quotes(line):
     if isinstance(line, (list, tuple)):
         line = ' '.join(line)
@@ -446,6 +586,11 @@ def split_line_with_quotes(line):
     return re.findall(r'("[^"]*"|[^" ]+)', line)
 
 
+def free_attributes(key):
+    # TODO maybe laster?!
+    return key  # .lower()  # .replace('_', '')
+
+
 ########################################################################################################################
 class BaseSectionObject(ABC):
     """
@@ -469,14 +614,35 @@ class BaseSectionObject(ABC):
 
     __name__ = 'BaseSectionObject'
 
+    @abc.abstractmethod
+    def __init__(self, *args, **kwargs):
+        pass
+
     def get(self, key):
-        if isinstance(key, list):
-            return tuple([self.get(k) for k in key])
-        return self.to_dict_().get(key)
+        """
+        Get an attribute value by the attribute name.
+
+        Args:
+            key (str): name of the attribute.
+
+        Returns:
+            any: the attribute value
+        """
+        if isinstance(key, (list, tuple, set)):
+            return type(key)([self.get(k) for k in key])
+        return self.__getattribute__(free_attributes(key))
 
     def set(self, key, value):
-        assert key in self.to_dict_(), f'{key} not a Object attribute | {self}'
-        vars(self)[key] = value
+        """
+        Set an attribute value.
+
+        Args:
+            key (str): name of the attribute.
+            value (any): value for the attribute.
+        """
+        if not hasattr(self, key):
+            raise SwmmInputWarning(f'{key} not a Object attribute | {self}')
+        self.__setattr__(free_attributes(key), value)
 
     def __getitem__(self, key):
         return self.get(key)
@@ -491,7 +657,43 @@ class BaseSectionObject(ABC):
         Returns:
             dict:
         """
-        return vars(self).copy()
+        return vars(self)
+
+    @property
+    def attributes(self):
+        """
+        Get the attributes names for the object.
+
+        Returns:
+            tuple[str]: Attribute names for the object
+        """
+        return tuple(self.to_dict_().keys())
+
+    @property
+    def values(self):
+        """
+        Get the attributes values for the object.
+
+        Returns:
+            tuple[any]: Attribute values for the object
+        """
+        return tuple(self.to_dict_().values())
+
+    @property
+    def values_used(self):
+        """
+        Get only the used attributes values for the object.
+
+        Values which aren't used are set as `:obj:numpy.nan`
+
+        Returns:
+            tuple[any]: Attribute values used for the object
+        """
+        return tuple(v for v in self.values if not (isinstance(v, float) and isnan(v)))
+
+    def __iter__(self):
+        for k, v in self.to_dict_().items():
+            yield k, v
 
     def __repr__(self):
         return str(self)
@@ -500,12 +702,10 @@ class BaseSectionObject(ABC):
         return self._to_debug_string()
 
     def __eq__(self, other):
-        # TODO: testing!!!
-        return isinstance(self, type(other)) and all([is_equal(self[k], other[k]) for k in self.to_dict_().keys()])
+        return isinstance(self, type(other)) and all([is_equal(self[k], other[k]) for k in self.attributes])
 
     def __hash__(self):
-        di = self.to_dict_()
-        return tuple([(k, v) for k, v in di.items()]).__hash__()
+        return tuple([(k, v) for k, v in self]).__hash__()
 
     # @property
     # def id(self):
@@ -520,42 +720,25 @@ class BaseSectionObject(ABC):
         Returns:
             str: debug string of the object
         """
-        args = list()
-        for k, d in self.to_dict_().items():
-            if isinstance(d, float) and isnan(d):
-                args.append(f'{k}=NaN')
-            elif isinstance(d, str):
-                args.append(f'{k}="{d}"')
-            else:
-                args.append(f'{k}={d}')
-        return '{}({})'.format(self.__class__.__name__, ', '.join(args))
+        return f'{self.__class__.__name__}({", ".join([f"{k}={repr(v)}" for k, v in self])})'
 
     def to_inp_line(self):
         """
-        convert object to one line of the ``.inp``-file
+        Convert object to one line of the ``.inp``-file.
 
         for ``.inp``-file writing
 
         Returns:
             str: SWMM .inp file compatible string
         """
-        di = self.to_dict_()
-        # s = ''
-        # if isinstance(self._identifier, list):
-        #     s += ' '.join([str(di.pop(i)) for i in self._identifier])
-        # else:
-        #     s += str(di.pop(self._identifier))
-
-        # s += ' ' + ' '.join([type2str(i) for i in di.values()])
-        # return s
-        return ' '.join([type2str(i) for i in di.values()])
+        return ' '.join([type2str(i) for i in self.values_used]).strip()
 
     @classmethod
     def from_inp_line(cls, *line_args):
         """
-        convert line in the ``.inp``-file to the object
+        Convert line in the ``.inp``-file to the object.
 
-        needed if multiple sub-classes of an object are available (i.e. Infiltration)
+        Needed if multiple subclasses of an object are available (i.e. Infiltration).
 
         Args:
             *line_args (list[str]): arguments in the line
@@ -563,7 +746,10 @@ class BaseSectionObject(ABC):
         Returns:
             BaseSectionObject: object of the ``.inp``-file section
         """
-        return cls(*line_args)
+        try:
+            return cls(*line_args)
+        except TypeError as e:
+            raise TypeError(f'{e} | {cls.__name__}{line_args}\n\n__init__{signature(cls.__init__)}\n\n{getdoc(cls.__init__)}')
 
     def copy(self):
         """
@@ -572,18 +758,24 @@ class BaseSectionObject(ABC):
         Returns:
             BaseSectionObject or Child: copy of the object
         """
-        return type(self)(**vars(self).copy())
+        # return type(self)(*self.values_used)
+        try:
+            return type(self)(**self.to_dict_())
+        except TypeError as e:
+            raise SwmmInputWarning(f'{e} | {self}')
 
     @classmethod
-    def create_section(cls, lines=None) -> InpSection:
+    def create_section(cls, lines=None):
         """
-        creates a new section for the ``.inp``-file of this object and ads objects described in `lines`
+        Create a new section for the ``.inp``-file of this object and adds objects described in `lines`
+
+        An empty section will be created when no lines are given.
 
         Args:
-            lines:
+            lines (list[list] | optional): lines of values for multiple objects in this section
 
         Returns:
-            InpSection: new section of this object
+            InpSection: new section of this object type
         """
         sec = cls._section_class(cls)
         # import sys
@@ -609,6 +801,15 @@ class BaseSectionObject(ABC):
 
     @classmethod
     def from_inp_lines(cls, lines):
+        """
+        Create a new section for the ``.inp``-file of this object and adds objects described in `lines`
+
+        Args:
+            lines (list[list]): lines of values for multiple objects in this section
+
+        Returns:
+            InpSection: new section of this object type
+        """
         return cls.create_section(lines)
 
     @classmethod
@@ -641,24 +842,26 @@ def dataframe_to_inp_string(df):
     Returns:
         str: .inp file conform string for one section
     """
-    comment_sign = ';;'
     if df.empty:
-        return ';; NO data'
+        return COMMENT_EMPTY_SECTION
+
+    if isinstance(df, Series):
+        return df.apply(type2str).to_string()
 
     c = df.copy()
     if c.columns.name is None:
-        c.columns.name = comment_sign
+        c.columns.name = COMMENT_STR
     else:
-        if not c.columns.name.startswith(comment_sign):
-            c.columns.name = comment_sign + c.columns.name
+        if not c.columns.name.startswith(COMMENT_STR):
+            c.columns.name = COMMENT_STR + c.columns.name
 
     if c.index.name is not None:
-        if not c.index.name.startswith(comment_sign):
-            c.index.name = comment_sign + c.index.name
+        if not c.index.name.startswith(COMMENT_STR):
+            c.index.name = COMMENT_STR + c.index.name
 
     if c.index._typ == 'multiindex':
         if c.index.names is not None:
-            if not c.index.levels[0].name.startswith(comment_sign):
+            if not c.index.levels[0].name.startswith(COMMENT_STR):
                 c.index.set_names(';' + c.index.names[0], level=0, inplace=True)
                 # because pandas 1.0
                 # c.index.levels[0].name = ';' + c.index.levels[0].name
@@ -690,12 +893,17 @@ def convert_section(head, lines, converter):
             return section_(lines)
 
         elif isclass(section_):  # section_ ... type/class
-            return section_.from_inp_lines(lines)
+            try:
+                return section_.from_inp_lines(lines)
+            except ValueError as e:
+                raise SwmmInputWarning(str(e) + f'\n{head}\n{section_}')
 
         else:
-            raise NotImplemented()
+            warnings.warn(f'Type of converter ({type(section_)}) for Section "{head}" not implemented. Section will not be converted.', SwmmInputWarning)
     else:
-        return lines.replace(SEP_INP, '').strip()
+        warnings.warn(f'Section "{head}" not implemented. Section will not be converted.', SwmmInputWarning)
+
+    return lines.replace(SEP_INP, '').strip()
 
 
 ########################################################################################################################
@@ -803,7 +1011,10 @@ re_int = re.compile('(\d+)')
 
 
 def natural_keys(text):
-    return [int(text) if text.isdigit() else text for text in re_int.split(text)]
+    if isinstance(text, str):
+        return [int(text) if text.isdigit() else text for text in re_int.split(text)]
+    else:
+        return [*(natural_keys(t) for t in text)]
 
 
 def section_to_string(section, fast=True, sort_objects_alphabetical=False):
@@ -820,39 +1031,61 @@ def section_to_string(section, fast=True, sort_objects_alphabetical=False):
     Returns:
         str: string of the ``.inp``-file section
     """
-    f = ''
-
-    # ----------------------
     if isinstance(section, str):  # Title
-        f += section.replace(SEP_INP, '').strip()
+        return section.replace(SEP_INP, '').strip()
 
     # ----------------------
-    elif isinstance(section, list):  # V0.1
+    elif isinstance(section, list):  # V1
+        return '\n'.join([type2str(line) for line in section])
+
+    # ----------------------
+    elif isinstance(section, dict):  # V2
+        max_len = len(max(section.keys(), key=len)) + 2
+        f = ''
+        for sub in section:
+            f += '{key}{value}'.format(key=sub.ljust(max_len), value=type2str(section[sub]) + '\n')
+        return f
+
+    # ----------------------
+    elif isinstance(section, (DataFrame, Series)):  # V3
+        return dataframe_to_inp_string(section)
+
+    # ----------------------
+    elif isinstance(section, InpSectionGeneric):  # V4
+        return section.to_inp_lines(fast=fast)
+
+    elif isinstance(section, InpSection):  # V4
+        return section.to_inp_lines(fast=fast, sort_objects_alphabetical=sort_objects_alphabetical)
+
+    elif section is None:
+        return ''
+
+    else:
+        print(section, '?'*10)
+
+
+def iter_section_lines(section, sort_objects_alphabetical=False):
+    if isinstance(section, str):  # Title
+        yield section.replace(SEP_INP, '').strip()
+
+    # ----------------------
+    elif isinstance(section, list):  # V1
         for line in section:
-            f += type2str(line) + '\n'
-    #
-    # # ----------------------
-    elif isinstance(section, dict):  # V0.2
+            yield type2str(line)
+
+    # ----------------------
+    elif isinstance(section, dict):  # V2
         max_len = len(max(section.keys(), key=len)) + 2
         for sub in section:
-            f += '{key}{value}'.format(key=sub.ljust(max_len),
-                                       value=type2str(section[sub]) + '\n')
-    #
-    # ----------------------
-    elif isinstance(section, (DataFrame, Series)):  # V0.3
-        if section.empty:
-            f += ';; NO data'
-
-        if isinstance(section, DataFrame):
-            f += dataframe_to_inp_string(section)
-
-        elif isinstance(section, Series):
-            f += section.apply(type2str).to_string()
+            yield'{key}{value}'.format(key=sub.ljust(max_len), value=type2str(section[sub]) + '\n')
 
     # ----------------------
-    elif isinstance(section, (InpSection, InpSectionGeneric)):  # V0.4
-        f += section.to_inp_lines(fast=fast, sort_objects_alphabetical=sort_objects_alphabetical)
+    elif isinstance(section, (DataFrame, Series)):  # V3
+        yield dataframe_to_inp_string(section)
 
     # ----------------------
-    f += '\n'
-    return f
+    elif isinstance(section, InpSectionGeneric):
+        yield section.to_inp_lines(fast=True)
+    elif isinstance(section, InpSection):  # V4
+        for line in section.iter_inp_lines(sort_objects_alphabetical=sort_objects_alphabetical):
+            yield line
