@@ -2,132 +2,259 @@ import os
 import re
 import warnings
 
-from .helpers import (section_to_string, CustomDictWithAttributes, convert_section, SEP_INP, InpSection,
-                      InpSectionGeneric, SECTION_ORDER_DEFAULT, check_order, )
+from .helpers import (section_to_string, CustomDict, convert_section, InpSection,
+                      InpSectionGeneric, SECTION_ORDER_DEFAULT, check_order, SECTIONS_ORDER_MP, head_to_str,
+                      iter_section_lines, SwmmInputWarning, BaseSectionObject, )
+from .._read_txt import detect_encoding, read_txt_file
 from .section_types import SECTION_TYPES
 from .section_labels import *
 from .sections import *
 from .sections.subcatch import INFILTRATION_DICT
 
 
-class SwmmInput(CustomDictWithAttributes):
+DEFAULT_ENCODING = 'utf-8'
+
+
+class SwmmInput(CustomDict):
     """
-    overall class for an input file
+    Overall class for an input file.
 
-    child class of dict
+    Child class of dict.
 
-    just used for the copy function and to identify ``.inp``-file data
+    Just used for the copy function and to identify ``.inp``-file data.
     """
 
-    def __init__(self, *args, custom_section_handler=None, **kwargs):
-        CustomDictWithAttributes.__init__(self, *args, **kwargs)
+    def __init__(self, *args, custom_section_handler=None, encoding=DEFAULT_ENCODING, force_ignore_case=False, **kwargs):
+        """
+
+        Args:
+            *args: only for creating the inp-object as a dict.
+            custom_section_handler:
+            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
+            **kwargs: only for creating the inp-object as a dict.
+        """
+        filename = None
+        if len(args) == 1 and isinstance(args[0], str) and os.path.isfile(args[0]):
+            # argument is an inp-file.
+            super().__init__()
+            filename = args[0]
+        else:
+            super().__init__(*args, **kwargs)
         self._converter = SECTION_TYPES.copy()
 
+        self._default_encoding = encoding
+
         if custom_section_handler is not None:
             self._converter.update(custom_section_handler)
 
         # only when reading a new file
-        self._original_section_order = None
+        self._original_section_order = SECTIONS_ORDER_MP
+
+        if filename is not None:
+            self._init_from_file(filename, force_ignore_case=force_ignore_case, encoding=encoding)
+
+    def copy(self):
+        """Copy inp-data."""
+        new = type(self)()
+        for key in self:
+            if hasattr(self._data[key], 'copy'):
+                new._data[key] = self._data[key].copy()
+            else:
+                new._data[key] = self._data[key]
+        return new
 
     def update(self, d=None, **kwargs):
+        """Update inp-data with another inp-data-object."""
         for sec in d:
             if sec not in self:
-                self[sec] = d[sec]
+                self._data[sec] = d._data[sec]
             else:
-                if isinstance(self[sec], str):
-                    pass
+                if isinstance(self._data[sec], str):
+                    if isinstance(d._data[sec], str):
+                        self._data[sec] += '\n' + d._data[sec]
+                    else:
+                        self[sec].update(d[sec])
+                        # warnings.warn(f'Updating of string section in INP-file not implemented! Skip Section {sec}')
                 else:
                     self[sec].update(d[sec])
 
-    @classmethod
-    def read_file(cls, filename, custom_converter=None, force_ignore_case=False, encoding='iso-8859-1'):
-        """
-        read ``.inp``-file and convert the sections in pythonic objects
+    def _init_from_str(self, txt, force_ignore_case=False):
+        # __________________________________
+        if force_ignore_case:
+            txt = txt.upper()
 
-        the sections will be converted when used
+        # __________________________________
+        self._original_section_order = []
+        for head, lines in zip(re.findall(r'\[(\w+)\]', txt),
+                               re.split(r'\[\w+\]', txt)[1:]):
+            head = head.upper()
+            if head in self._data:
+                self._data[head] += '\n' + lines.strip()
+            else:
+                self._data[head] = lines.strip()
+                self._original_section_order.append(head)
+
+        # ----------------
+        # if order in inp follows default SWMM GUI order
+        #   set full/complete order list of SWMM GUI
+        #   to use the right order for additional created sections
+        if check_order(self, SECTION_ORDER_DEFAULT):
+            self._original_section_order = SECTION_ORDER_DEFAULT
+
+        self.set_default_infiltration_from_options()
+
+    def _init_from_file(self, filename, force_ignore_case=False, encoding=DEFAULT_ENCODING):
+        """
+        Read ``.inp``-file and convert the sections in pythonic objects.
+
+        The sections will be converted when used.
+
+        Args:
+            filename (str): path/filename to .inp file
+            force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
+                                        all text/labels will be set to uppercase
+            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
+        """
+        if encoding is None:
+            encoding = detect_encoding(filename)
+
+        self._default_encoding = encoding
+
+        if os.path.isfile(filename) or filename.endswith('.inp'):
+            txt = read_txt_file(filename, encoding=encoding)
+
+        else:
+            warnings.warn('Reading a string with SwmmInput.read_file is deprecated. Use SwmmInput.read_text instead.', DeprecationWarning)
+            txt = filename
+
+        self._init_from_str(txt, force_ignore_case=force_ignore_case)
+
+    @classmethod
+    def read_file(cls, filename, custom_converter=None, force_ignore_case=False, encoding=DEFAULT_ENCODING):
+        """
+        Read ``.inp``-file and convert the sections in pythonic objects.
+
+        The sections will be converted when used.
 
         Args:
             filename (str): path/filename to .inp file
             custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
             force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
                                         all text/labels will be set to uppercase
-            encoding (str): encoding of the inp text file
+            encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
 
         Returns:
             SwmmInput: dict-like data of the sections in the ``.inp``-file
         """
         inp = cls(custom_section_handler=custom_converter)
-        if os.path.isfile(filename) or filename.endswith('.inp'):
-            with open(filename, 'r', encoding=encoding) as inp_file:
-                txt = inp_file.read()
-        else:
-            txt = filename
-
-        # __________________________________
-        if force_ignore_case:
-            txt = txt.upper()
-
-        # __________________________________
-        inp._original_section_order = []
-        for head, lines in zip(re.findall(r"\[(\w+)\]", txt),
-                               re.split(r"\[\w+\]", txt)[1:]):
-            inp._data[head.upper()] = lines.strip()
-            inp._original_section_order.append(head.upper())
-
-        # ----------------
-        # if order in inp follows default SWMM GUI order
-        #   set full/complete order list of SWMM GUI
-        #   to use the right order for additional created setions
-        if check_order(inp, SECTION_ORDER_DEFAULT):
-            inp._original_section_order = SECTION_ORDER_DEFAULT
-
-        inp.set_default_infiltration_from_options()
+        inp._init_from_file(filename, force_ignore_case, encoding)
         return inp
 
+    @classmethod
+    def read_text(cls, txt, custom_converter=None, force_ignore_case=False):
+        """
+        Read the text of an ``.inp``-file and convert the sections in pythonic objects.
+
+        The sections will be converted when used.
+
+        Args:
+            txt (str): text of the .inp file
+            custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
+            force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
+                                        all text/labels will be set to uppercase
+
+        Returns:
+            SwmmInput: dict-like data of the sections in the ``.inp``-file
+        """
+        inp = cls(custom_section_handler=custom_converter)
+        inp._init_from_str(txt, force_ignore_case=force_ignore_case)
+
     def force_convert_all(self):
-        _ = [self[sec] for sec in self]
+        """
+        Convert all sections to pythonic objects.
+
+        By default, unused sections will be stored as string.
+
+        Necessary to reduce final file-size with keyword (fast=True).
+        """
+        for key in self:
+            self._convert_section(key)
 
     def __getitem__(self, key):
         # if section not in inp-data, create an empty section
         if key not in self:
             self._data[key] = self._converter[key].create_section()
-
-        # if section is a string (raw string from the .inp-file) convert section first
-        if isinstance(self._data[key], str):
-            self[key] = convert_section(key, self._data[key], self._converter)
+        else:
+            # if section is a string (raw string from the .inp-file) convert section first
+            self._convert_section(key)
 
         return self._data.__getitem__(key)
 
     def __setitem__(self, key, item):
         super().__setitem__(key, item)
-        # super().__setattr__(key, item)
-        # self._data.__setitem__(key, item)
-        # if key == OPTIONS:
-        #     self.set_default_infiltration_from_options()
-        if hasattr(self[key], 'set_parent_inp'):
-            self[key].set_parent_inp(self)
+        # if a new section is added, make the section aware in which inp file it is.
+        if hasattr(self._data[key], 'set_parent_inp'):
+            self._data[key].set_parent_inp(self)
 
-    def __delattr__(self, item):
+    def __delattr__(self, attribute_name):
         """delete section"""
-        if item in SECTION_TYPES:
-            # print('delete section')
-            del self._data[item]
-        else:
-            # print('delete attribute')
-            super().__delattr__(item)
+        # if the attribute_name is a known section-key then only delete the data (not the attribute)
+        if attribute_name in self._data.keys():
+            del self._data[attribute_name]
+        else:  # else delete the attribute
+            super().__delattr__(attribute_name)
+
+    def _convert_section(self, key):
+        # if section is a string (raw string from the .inp-file) convert section first
+        if isinstance(self._data[key], str):
+            self._data[key] = convert_section(key, self._data[key], self._converter)
+
+            if hasattr(self._data[key], 'set_parent_inp'):
+                self._data[key].set_parent_inp(self)
 
     def set_default_infiltration_from_options(self):
+        """Set the default infiltration class based on the OPTIONS section."""
         if OPTIONS in self \
                 and 'INFILTRATION' in self[OPTIONS] \
                 and isinstance(self[OPTIONS], (dict, OptionSection, InpSectionGeneric)):
             self.set_infiltration_method(INFILTRATION_DICT.get(self[OPTIONS]['INFILTRATION']))
 
     def set_infiltration_method(self, infiltration_class):
+        """
+        Set the default infiltration class.
+
+        Args:
+            infiltration_class: One of
+                :class:`~swmm_api.input_file.sections.InfiltrationCurve`,
+                :class:`~swmm_api.input_file.sections.NumberInfiltrationGreenAmpt`,
+                :class:`~swmm_api.input_file.sections.InfiltrationHorton`
+        """
         self._converter[INFILTRATION] = infiltration_class
 
+    def _get_section_headers(self, custom_sections_order=None):
+        """
+        Get list of section keys/headers/labels.
+
+        Args:
+            custom_sections_order (list): Custom list for section sorting. (optional)
+
+        Returns:
+            list: sorted section-headers based on given order
+        """
+        if custom_sections_order is None:
+            custom_sections_order = self._original_section_order
+
+        def _sort_by(key):
+            if key in custom_sections_order:
+                return custom_sections_order.index(key)
+            else:
+                return len(custom_sections_order)
+
+        return sorted(self.keys(), key=_sort_by)
+
     def to_string(self, fast=True, custom_sections_order=None, sort_objects_alphabetical=False):
         """
-        create the string of a new ``.inp``-file
+        Convert the inp-data to a ``.inp``-file-string.
 
         Args:
             fast (bool): don't use any formatting else format as table
@@ -140,25 +267,15 @@ class SwmmInput(CustomDictWithAttributes):
             str: string of input file text
         """
         f = ''
-        sep = f'\n{SEP_INP}\n[{{}}]\n'
-
-        if custom_sections_order is None:
-            custom_sections_order = self._original_section_order
-
-        def _sort_by(key):
-            if key in custom_sections_order:
-                return custom_sections_order.index(key)
-            else:
-                return len(custom_sections_order)
-
-        for head in sorted(self.keys(), key=_sort_by):
-            f += sep.format(head)
+        for head in self._get_section_headers(custom_sections_order):
+            f += head_to_str(head)
             f += section_to_string(self._data[head], fast=fast, sort_objects_alphabetical=sort_objects_alphabetical)
         return f
 
-    def write_file(self, filename, fast=True, encoding='iso-8859-1', custom_sections_order=None, sort_objects_alphabetical=False):
+    def write_file(self, filename, fast=True, encoding=None, custom_sections_order=None,
+                   sort_objects_alphabetical=False, per_line=False):
         """
-        create/write a new ``.inp``-file
+        Write a new ``.inp``-file.
 
         Args:
             filename (str): path/filename of created ``.inp``-file
@@ -168,24 +285,63 @@ class SwmmInput(CustomDictWithAttributes):
                 inp-file | default: order of the read inp-file + default order of the SWMM GUI
             sort_objects_alphabetical (bool): if objects in a section should be sorted alphabetical |
                 default: use order of the read inp-file and append new objects
+            per_line (bool): weather to write the data line per line (=True) or section per section (=False) into the
+                file. line per line has an advantage for big files (> 1 GB) and uses less memory (RAM).
         """
+
+        if encoding is None:
+            encoding = self._default_encoding
+
+        if encoding is None:
+            encoding = DEFAULT_ENCODING
+
         with open(filename, 'w', encoding=encoding) as f:
-            f.write(self.to_string(fast=fast, custom_sections_order=custom_sections_order))
+            for head in self._get_section_headers(custom_sections_order):
+                if not self._data[head]:  # if section is empty
+                    continue
+                f.write(head_to_str(head))
+                if per_line:
+                    for line in iter_section_lines(self._data[head],
+                                                   sort_objects_alphabetical=sort_objects_alphabetical):
+                        f.write(line + '\n')
+                else:
+                    f.write(section_to_string(self._data[head], fast=fast,
+                                              sort_objects_alphabetical=sort_objects_alphabetical))
         return filename
 
+    def print_string(self, custom_sections_order=None):
+        """
+        Print the string of the inp-data (to the stdout).
+
+        Args:
+            custom_sections_order (list[str]): list of section names to preset the order of the section in the created
+                inp-file | default: order of the read inp-file + default order of the SWMM GUI
+        """
+        for head in self._get_section_headers(custom_sections_order):
+            print(head_to_str(head))
+            for line in iter_section_lines(self._data[head], sort_objects_alphabetical=False):
+                print(line)
+
     def check_for_section(self, obj):
         """
-        check if a section is in the inp-data, and create it if not present
+        Check if a section is in the inp-data, and create it if not present.
 
         Args:
-            obj (BaseSectionObject or InpSectionGeneric):
+            obj (BaseSectionObject or InpSectionGeneric): section object
 
         Returns:
-            InpSection or InpSectionGeneric: section of inp
+            swmm_api.input_file.helpers.InpSection | swmm_api.input_file.helpers.InpSectionGeneric: section of inp
         """
-        if obj._section_label not in self:
-            self[obj._section_label] = obj.create_section()
-        return self[obj._section_label]
+        if hasattr(obj, '_section_label'):
+            sec = obj._section_label
+        elif hasattr(obj, '_label'):
+            sec = obj._label
+        else:
+            warnings.warn(f'Unknown Section Object type "{type(obj)}"', SwmmInputWarning)
+
+        if sec not in self:
+            self[sec] = obj.create_section()
+        return self[sec]
 
     def add_new_section(self, section):
         """
@@ -200,7 +356,7 @@ class SwmmInput(CustomDictWithAttributes):
         if section._label not in self:
             self[section._label] = section
         else:
-            warnings.warn(f'Section [{section._label}] not empty!')
+            warnings.warn(f'Section [{section._label}] not empty!', SwmmInputWarning)
 
     def add_obj(self, obj):
         """
@@ -223,37 +379,91 @@ class SwmmInput(CustomDictWithAttributes):
             self.add_obj(obj)
 
     @property
-    def OPTIONS(self) -> OptionSection:
+    def TITLE(self):
+        """
+        TITLE Section
+
+        Returns:
+            TitleSection: TITLE Section
+        """
+        if TITLE in self:
+            return self[TITLE]
+
+    @property
+    def OPTIONS(self):
+        """
+        OPTIONS Section
+
+        Returns:
+            OptionSection: OPTIONS Section
+        """
         if OPTIONS in self:
             return self[OPTIONS]
 
     @property
-    def REPORT(self) -> ReportSection:
-        if REPORT in self:
-            return self[REPORT]
+    def REPORT(self):
+        """
+        REPORT Section
+
+        Returns:
+            ReportSection: REPORT Section
+        """
+        if REPORT not in self:
+            self[REPORT] = ReportSection()
+        return self[REPORT]
 
     @property
-    def EVAPORATION(self) -> EvaporationSection:
+    def EVAPORATION(self):
+        """
+        EVAPORATION Section
+
+        Returns:
+            EvaporationSection: EVAPORATION Section
+        """
         if EVAPORATION in self:
             return self[EVAPORATION]
 
     @property
-    def TEMPERATURE(self) -> TemperatureSection:
+    def TEMPERATURE(self):
+        """
+        TEMPERATURE Section
+
+        Returns:
+            TemperatureSection: TEMPERATURE Section
+        """
         if TEMPERATURE in self:
             return self[TEMPERATURE]
 
     @property
-    def FILES(self) -> FilesSection:
+    def FILES(self):
+        """
+        FILES Section
+
+        Returns:
+            FilesSection: FILES Section
+        """
         if FILES in self:
             return self[FILES]
 
     @property
-    def BACKDROP(self) -> BackdropSection:
+    def BACKDROP(self):
+        """
+        BACKDROP Section
+
+        Returns:
+            BackdropSection: BACKDROP Section
+        """
         if BACKDROP in self:
             return self[BACKDROP]
 
     @property
-    def ADJUSTMENTS(self) -> AdjustmentsSection:
+    def ADJUSTMENTS(self):
+        """
+        ADJUSTMENTS Section
+
+        Returns:
+            AdjustmentsSection: ADJUSTMENTS Section
+        """
         if ADJUSTMENTS in self:
             return self[ADJUSTMENTS]
 
@@ -264,7 +474,7 @@ class SwmmInput(CustomDictWithAttributes):
         COORDINATES section
 
         Returns:
-            dict[str, Coordinate | CoordinateGeo] | InpSection: Coordinates in INP
+            dict[str, Coordinate] | InpSectionGeo: Coordinates in INP
         """
         if COORDINATES in self:
             return self[COORDINATES]
@@ -275,7 +485,7 @@ class SwmmInput(CustomDictWithAttributes):
         VERTICES section
 
         Returns:
-            dict[str, Vertices] | InpSection: Vertices section
+            dict[str, Vertices] | InpSectionGeo: Vertices section
         """
         if VERTICES in self:
             return self[VERTICES]
@@ -286,7 +496,7 @@ class SwmmInput(CustomDictWithAttributes):
         POLYGONS section
 
         Returns:
-            dict[str, Polygon] | InpSection: Polygon section
+            dict[str, Polygon] | InpSectionGeo: Polygon section
         """
         if POLYGONS in self:
             return self[POLYGONS]
@@ -732,20 +942,24 @@ class SwmmInput(CustomDictWithAttributes):
             return self[LID_USAGE]
 
 
-def read_inp_file(filename, custom_converter=None, force_ignore_case=False, encoding='iso-8859-1'):
+def read_inp_file(filename, custom_converter=None, force_ignore_case=False, encoding=DEFAULT_ENCODING):
     """
-    read ``.inp``-file and convert the sections in pythonic objects
+    Read ``.inp``-file and convert the sections in pythonic objects.
 
-    the sections will be converted when used
+    The sections will be converted when used.
 
     Args:
         filename (str): path/filename to .inp file
         custom_converter (dict): dictionary of {section: converter/section_type} Default: :py:const:`SECTION_TYPES`
         force_ignore_case (bool): SWMM is case-insensitive but python is case-sensitive -> set True to ignore case
-        encoding (str): encoding of the inp text file
+                                    all text/labels will be set to uppercase
+        encoding (str): encoding of the .inp-text-file (None -> auto-detect encoding ... takes a few seconds)
 
     Returns:
         SwmmInput: dict-like data of the sections in the ``.inp``-file
+
+    See Also:
+        :meth:`SwmmInput.read_file` : Equal functionality.
     """
     return SwmmInput.read_file(filename, custom_converter=custom_converter, force_ignore_case=force_ignore_case,
                                encoding=encoding)
