@@ -1,11 +1,12 @@
 from networkx import DiGraph, all_simple_paths, subgraph, node_connected_component
 
-from ..inp import SwmmInput
 from .collection import nodes_dict, links_dict
 from .filter import create_sub_inp
+from ..inp import SwmmInput
+from ..section_labels import SUBCATCHMENTS
 
 
-def inp_to_graph(inp: SwmmInput) -> DiGraph:
+def inp_to_graph(inp):
     """
     create a network of the model with the networkx package
 
@@ -24,7 +25,15 @@ def inp_to_graph(inp: SwmmInput) -> DiGraph:
         #     g.add_node(link.FromNode)
         # if link.ToNode not in g:
         #     g.add_node(link.ToNode)
-        g.add_edge(link.FromNode, link.ToNode, label=link.Name)
+        if g.has_edge(link.from_node, link.to_node):
+            old_label = g.get_edge_data(link.from_node, link.to_node)['label']
+            if isinstance(old_label, str):
+                new_label = [old_label, link.name]
+            elif isinstance(old_label, list):
+                new_label.append(link.name)
+            g.add_edge(link.from_node, link.to_node, label=new_label)
+        else:
+            g.add_edge(link.from_node, link.to_node, label=link.name)
     return g
 
 
@@ -34,10 +43,7 @@ def get_path(g, start, end):
 
 
 def get_path_subgraph(base, start, end):
-    if isinstance(base, SwmmInput):
-        g = inp_to_graph(base)
-    else:
-        g = base
+    g = inp_to_graph(base) if isinstance(base, SwmmInput) else base
     sub_list = get_path(g, start=start, end=end)
     sub_graph = subgraph(g, sub_list)
     return sub_list, sub_graph
@@ -47,15 +53,21 @@ def next_links(inp, node, g=None):
     if g is None:
         g = inp_to_graph(inp)
     links = links_dict(inp)
+    for label in _next_links_labels(g, node):
+        yield links[label]
+
+
+def _next_links_labels(g, node):
     for i in g.out_edges(node):
-        yield links[g.get_edge_data(*i)['label']]
+        label = g.get_edge_data(*i)['label']
+        if isinstance(label, list):
+            yield from label
+        else:
+            yield label
 
 
 def next_links_labels(g, node):
-    labels = []
-    for i in g.out_edges(node):
-        labels.append(g.get_edge_data(*i)['label'])
-    return labels
+    return list(_next_links_labels(g, node))
 
 
 def next_nodes(g, node):
@@ -66,15 +78,21 @@ def previous_links(inp, node, g=None):
     if g is None:
         g = inp_to_graph(inp)
     links = links_dict(inp)
+    for label in _previous_links_labels(g, node):
+        yield links[label]
+
+
+def _previous_links_labels(g, node):
     for i in g.in_edges(node):
-        yield links[g.get_edge_data(*i)['label']]
+        label = g.get_edge_data(*i)['label']
+        if isinstance(label, list):
+            yield from label
+        else:
+            yield label
 
 
 def previous_links_labels(g, node):
-    labels = []
-    for i in g.in_edges(node):
-        labels.append(g.get_edge_data(*i)['label'])
-    return labels
+    return list(_previous_links_labels(g, node))
 
 
 def previous_nodes(g, node):
@@ -82,15 +100,21 @@ def previous_nodes(g, node):
 
 
 def links_connected(inp, node, g=None):
+    """
+    Get the names of the links connected to the node.
+
+    Args:
+        inp (SwmmInput): inp-data.
+        node (str): name of the node.
+        g (networkx.DiGraph | optionl): graph of the swmm-network.
+
+    Returns:
+        tuple[list[swmm_api.input_file.sections.link._Link]]: list of the upstream and downstream links connected to the node
+    """
     if g is None:
         g = inp_to_graph(inp)
-    links = links_dict(inp)
-    next_ = []
-    for i in g.out_edges(node):
-        next_.append(links[g.get_edge_data(*i)['label']])
-    previous_ = []
-    for i in g.in_edges(node):
-        previous_.append(links[g.get_edge_data(*i)['label']])
+    next_ = list(next_links(inp, node, g=g))
+    previous_ = list(previous_links(inp, node, g=g))
     return previous_, next_
 
 
@@ -98,7 +122,7 @@ def number_in_out(g, node):
     return len(list(g.predecessors(node))), len(list(g.successors(node)))
 
 
-def downstream_nodes(graph: DiGraph, node: str) -> list:
+def downstream_nodes(graph, node):
     """
     get all nodes downstream of the node given
 
@@ -116,7 +140,7 @@ def downstream_nodes(graph: DiGraph, node: str) -> list:
 
 def _downstream_nodes(graph: DiGraph, node: str, node_list=None) -> list:
     if node_list is None:
-        node_list = list()
+        node_list = []
     node_list.append(node)
     next_nodes = list(graph.successors(node))
     if next_nodes:
@@ -127,7 +151,7 @@ def _downstream_nodes(graph: DiGraph, node: str, node_list=None) -> list:
     return node_list
 
 
-def upstream_nodes(graph: DiGraph, node: str) -> list:
+def upstream_nodes(graph, node):
     """
     get all nodes upstream of the node given
 
@@ -145,7 +169,7 @@ def upstream_nodes(graph: DiGraph, node: str) -> list:
 
 def _upstream_nodes(graph: DiGraph, node: str, node_list=None) -> list:
     if node_list is None:
-        node_list = list()
+        node_list = []
     node_list.append(node)
     next_nodes = list(graph.predecessors(node))
     if next_nodes:
@@ -160,10 +184,7 @@ def get_network_forks(inp):
     # pd.DataFrame.from_dict(forks, orient='index')
     g = inp_to_graph(inp)
     nodes = nodes_dict(inp)
-    forks = dict()
-    for n in nodes:
-        forks[n] = number_in_out(g, n)
-    return forks
+    return {n: number_in_out(g, n) for n in nodes}
 
 
 def split_network(inp, keep_node, split_at_node=None, keep_split_node=True, graph=None, init_print=True):
@@ -240,6 +261,7 @@ def conduit_iter_over_inp(inp, start, end):
         for n in shortest_path_nodes:
             for i in g.out_edges(n):
                 if i[1] in shortest_path_nodes:
+                    # TODO multiple labels
                     yield g.get_edge_data(*i)['label']
     # else:
     #     node = start
@@ -261,4 +283,46 @@ def conduit_iter_over_inp(inp, start, end):
     #             found = True
     #             break
     #     if not found or (node is not None and (node == end)):
-    #         break
\ No newline at end of file
+    #         break
+
+
+def inp_to_graph2(inp):
+    """
+    Create a network of the model with the networkx package.
+
+    Args:
+        inp (SwmmInput): inp-file data
+
+    Returns:
+        networkx.DiGraph: networkx graph of the model
+    """
+    # g = nx.Graph()
+    g = DiGraph()
+    for node_label, node in nodes_dict(inp).items():
+        g.add_node(node_label, obj=node)
+
+    if SUBCATCHMENTS in inp:
+        for sc_label, sc in inp[SUBCATCHMENTS].items():
+            g.add_node(sc.name, obj=sc)
+        for sc in inp.SUBCATCHMENTS.values():
+            g.add_edge(sc.name, sc.outlet, label=f'Outlet({sc.name})')
+
+    for link in links_dict(inp).values():
+        # if link.FromNode not in g:
+        #     g.add_node(link.FromNode)
+        # if link.ToNode not in g:
+        #     g.add_node(link.ToNode)
+        if g.has_edge(link.from_node, link.to_node):
+            old_label = g.get_edge_data(link.from_node, link.to_node)['label']
+            old_obj = g.get_edge_data(link.from_node, link.to_node)['obj']
+            if isinstance(old_label, str):
+                new_label = [old_label, link.name]
+                new_obj = [old_obj, link]
+            elif isinstance(old_label, list):
+                new_label.append(link.name)
+                new_obj.append(old_obj)
+            g.add_edge(link.from_node, link.to_node, label=new_label, obj=new_obj)
+            print(new_label)
+        else:
+            g.add_edge(link.from_node, link.to_node, label=link.name, obj=link)
+    return g
\ No newline at end of file
