@@ -1,15 +1,16 @@
-from ..section_labels import *
 from .collection import links_dict, nodes_dict
+from ._helpers import get_used_curves
+from ..section_labels import *
 from ..misc.curve_simplification import ramer_douglas
 from ..section_types import SECTION_TYPES
-from ..sections import Control
+from ..sections import Control, EvaporationSection
 
 
 def reduce_curves(inp):
     """
-    remove unused curves
+    Remove unused curves.
 
-    only keep used CURVES from the sections [STORAGE, OUTLETS, PUMPS and XSECTIONS]
+    Only keep used CURVES from the sections [STORAGE, OUTFALLS, OUTLETS, PUMPS and XSECTIONS].
 
     Args:
         inp (swmm_api.SwmmInput): inp-file data
@@ -19,13 +20,7 @@ def reduce_curves(inp):
     """
     if CURVES not in inp:
         return inp
-    used_curves = set()
-    for section in [STORAGE, OUTLETS, PUMPS, XSECTIONS]:
-        if section in inp:
-            for name in inp[section]:
-                if isinstance(inp[section][name].Curve, str):
-                    used_curves.add(inp[section][name].Curve)
-
+    used_curves = get_used_curves(inp)
     inp[CURVES] = inp[CURVES].slice_section(used_curves)
 
 
@@ -42,7 +37,7 @@ def reduce_pattern(inp):
 
     if INFLOWS in inp:
         #  optional time pattern used to adjust the baseline value on a periodic basis
-        used_pattern |= set(inp[INFLOWS].frame['Pattern'].dropna().values)
+        used_pattern |= set(inp[INFLOWS].frame['pattern'].dropna().values)
 
     if DWF in inp:
         for i in range(1, 5):
@@ -150,9 +145,10 @@ def reduce_raingages(inp):
     Returns:
         SwmmInput: inp-file data with filtered RAINGAGES section
     """
-    if SUBCATCHMENTS not in inp or RAINGAGES not in inp:
-        return inp
-    needed_raingages = {inp[SUBCATCHMENTS][s].RainGage for s in inp[SUBCATCHMENTS]}
+    needed_raingages = set()
+    if (SUBCATCHMENTS in inp) and (RAINGAGES in inp):
+        needed_raingages = {inp[SUBCATCHMENTS][s].rain_gage for s in inp[SUBCATCHMENTS]}
+
     inp[RAINGAGES] = inp[RAINGAGES].slice_section(needed_raingages)
 
 
@@ -169,3 +165,35 @@ def remove_empty_sections(inp):
     for section in list(inp.keys()):
         if not inp[section]:
             del inp[section]
+
+
+def reduce_timeseries(inp):
+    if TIMESERIES not in inp:
+        return
+
+    needed_timeseries = set()
+    key = EvaporationSection.KEYS.TIMESERIES  # TemperatureSection.KEYS.TIMESERIES, ...
+
+    if RAINGAGES in inp:
+        f = inp[RAINGAGES].frame
+        # type: Raingage
+        if not f.empty:
+            needed_timeseries |= set(f.loc[f['Source'].str.upper() == key, 'Timeseries'])
+
+    if EVAPORATION in inp:
+        if key in inp[EVAPORATION]:
+            needed_timeseries.add(inp[EVAPORATION][key])
+
+    if TEMPERATURE in inp:
+        if key in inp[TEMPERATURE]:
+            needed_timeseries.add(inp[TEMPERATURE][key])
+
+    if OUTFALLS in inp:
+        f = inp[OUTFALLS].frame
+        needed_timeseries |= set(f.loc[f['kind'].str.upper() == key, 'data'])
+
+    if INFLOWS in inp:
+        f = inp[INFLOWS].frame
+        needed_timeseries |= set(f['time_series'])
+
+    inp[TIMESERIES] = inp[TIMESERIES].slice_section(needed_timeseries)
