@@ -2,18 +2,20 @@ from itertools import product
 
 import pandas as pd
 
-from ... import SwmmInput
-from ..section_abr import SEC
+from ..inp import SwmmInput
+from ..section_labels import TAGS, SUBCATCHMENTS
 from .collection import nodes_dict, links_dict
 from ..sections import Tag
 
+TAG_COL_NAME = 'tag'
+
 
 def _get_tags_frame(inp, part=None):
-    if SEC.TAGS in inp:
-        df_tags = inp[SEC.TAGS].frame
+    if TAGS in inp and inp[TAGS]:
+        df_tags = inp[TAGS].frame
         if part in df_tags.index.levels[0]:
-            return inp[SEC.TAGS].frame.xs(part, axis=0, level=0)
-    return pd.Series(name='tags', dtype=str)
+            return inp[TAGS].frame.xs(part, axis=0, level=0)
+    return pd.Series(name=TAG_COL_NAME, dtype=str)
 
 
 def get_node_tags(inp):
@@ -75,7 +77,7 @@ def filter_tags(inp_tags: SwmmInput, inp_objects: SwmmInput = None):
     links = links_dict(inp_objects)
     keys += list(product([Tag.TYPES.Link], list(links.keys())))
 
-    if SEC.SUBCATCHMENTS in inp_objects:
+    if SUBCATCHMENTS in inp_objects:
         keys += list(product([Tag.TYPES.Subcatch], list(inp_objects.SUBCATCHMENTS.keys())))
 
     return inp_tags.TAGS.slice_section(keys)
@@ -89,7 +91,7 @@ def delete_tag_group(inp, part):
         inp (SwmmInput):
         part (str): label of the group i.e.: :attr:`Tag.TYPE.Node` (or the strings: 'Node', 'Subcatch', 'Link')
     """
-    if SEC.TAGS in inp:
-        for key in list(inp[SEC.TAGS].keys()):
-            if inp[SEC.TAGS][key].kind == part:
-                del inp[SEC.TAGS][key]
+    if TAGS in inp:
+        for key in list(inp[TAGS].keys()):
+            if inp[TAGS][key].kind == part:
+                del inp[TAGS][key]
